# Generated by roxygen2: do not edit by hand

export(eems)
export(eems.from.files)
export(eems.plots)
export(eems.population.grid)
export(eems.posterior.draws)
export(eems.voronoi.samples)
export(grid.make)
export(outer.buffered)
import(Rcpp)
import(grDevices)
import(graphics)
import(raster)
import(sf)
import(sp)
importFrom(stats,lm)
importFrom(stats,na.omit)
importFrom(utils,capture.output)
importFrom(utils,read.table)
importFrom(utils,write.table)
useDynLib(reems)
