#' Example allele frequency data from ConStruct
#'
#' @format ## `ex.freqs`
#' A matrix with 16 rows and 10000 columns.
#' The rows designate samples and the columns loci.
#' @source https://github.com/gbradburd/conStruct/tree/master/data
"ex.freqs"

#' Example coordinate data from ConStruct
#'
#' @format ## `ex.coords`
#' A matrix with 16 rows and 2 columns.
#' The rows designate samples and the two columns store the X and Y coordinate respectively. .
#' @source https://github.com/gbradburd/conStruct/tree/master/data
"ex.coords"
