% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{outer.buffered}
\alias{outer.buffered}
\title{Create a buffered habitat around a set of points.}
\usage{
outer.buffered(coords, d = NULL)
}
\arguments{
\item{coords}{A two-column matrix of point coordinates (longitude, latitude).}

\item{d}{Rough buffer distance. Not necessarily kept completely to keep the polygon simple.
Defaults to 0.2 of the length of the hull of the \code{coords}.}
}
\value{
A two-column matrix of coordinates with the outer boundary points of the containing polygon.
}
\description{
Creates a polygon a certain distance from each of a set of points.
It is used for creating a default habitat outline for \code{\link[=eems]{eems()}}.
}
\examples{
#  Create a buffer around the example habitat
outer <- outer.buffered(ex.coords)
}
