% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{ex.coords}
\alias{ex.coords}
\title{Example coordinate data from ConStruct}
\format{
\subsection{\code{ex.coords}}{

A matrix with 16 rows and 2 columns.
The rows designate samples and the two columns store the X and Y coordinate respectively. .
}
}
\source{
https://github.com/gbradburd/conStruct/tree/master/data
}
\usage{
ex.coords
}
\description{
Example coordinate data from ConStruct
}
\keyword{datasets}
